@@ -38,11 +38,6 @@ const guint THUMB_VIEW_MINIMUM_HEIGHT = 100;
 
 const gchar CHEESE_OLD_VIDEO_NAME_SUFFIX[] = ".ogv";
 
-#define CHEESE_THUMB_VIEW_GET_PRIVATE(o) \
-  (G_TYPE_INSTANCE_GET_PRIVATE ((o), CHEESE_TYPE_THUMB_VIEW, CheeseThumbViewPrivate))
-
-G_DEFINE_TYPE (CheeseThumbView, cheese_thumb_view, GTK_TYPE_ICON_VIEW);
-
 typedef struct
 {
   GtkListStore *store;
@@ -57,6 +52,8 @@ typedef struct
   GQueue *thumbnails;
 } CheeseThumbViewPrivate;
 
+G_DEFINE_TYPE_WITH_PRIVATE (CheeseThumbView, cheese_thumb_view, GTK_TYPE_ICON_VIEW);
+
 enum
 {
   THUMBNAIL_PIXBUF_COLUMN,
@@ -98,7 +95,7 @@ cheese_thumb_view_idle_append_item (gpointer data)
   if (item == NULL) return FALSE;
 
   thumb_view = item->thumb_view;
-  priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+  priv = cheese_thumb_view_get_instance_private (thumb_view);
 
 
   GnomeDesktopThumbnailFactory *factory = priv->factory;
@@ -189,7 +186,7 @@ cheese_thumb_view_idle_append_item (gpointer data)
 static void
 cheese_thumb_view_append_item (CheeseThumbView *thumb_view, GFile *file)
 {
-  CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+  CheeseThumbViewPrivate *priv = cheese_thumb_view_get_instance_private (thumb_view);
 
   GtkTreeIter   iter;
   GtkIconTheme *icon_theme;
@@ -265,14 +262,15 @@ cheese_thumb_view_append_item (CheeseThumbView *thumb_view, GFile *file)
     g_free (filename);
   }
 
-  if (priv->multiplex_thumbnail_generator)
-  {
-    char *f;
+    if (priv->multiplex_thumbnail_generator)
+    {
+        gchar *f;
 
-    f      = g_strdup_printf ("%s/pixmaps/cheese-%i.svg", PACKAGE_DATADIR, g_random_int_range (1, 4));
-    pixbuf = gdk_pixbuf_new_from_file (f, NULL);
-    g_free (f);
-  }
+        f = g_strdup_printf ("/org/gnome/Cheese/pixmaps/cheese-%i.svg",
+                             g_random_int_range (1, 4));
+        pixbuf = gdk_pixbuf_new_from_resource (f, NULL);
+        g_free (f);
+    }
   else
   {
     icon_theme = gtk_icon_theme_get_default ();
@@ -322,41 +320,54 @@ cheese_thumb_view_append_item (CheeseThumbView *thumb_view, GFile *file)
 void
 cheese_thumb_view_remove_item (CheeseThumbView *thumb_view, GFile *file)
 {
-  CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+    CheeseThumbViewPrivate *priv = cheese_thumb_view_get_instance_private (thumb_view);
 
-  char       *path;
-  GtkTreeIter iter;
-  char       *filename;
-  gboolean    found = FALSE;
+    gchar *filename;
+    GtkTreeIter iter;
+    gboolean found = FALSE;
 
-  filename = g_file_get_path (file);
+    filename = g_file_get_path (file);
 
-  if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->store), &iter))
-  {
-    /* a single item was on the thumbview but it's been already removed */
-    return;
-  }
-
-  /* check if the selected item is the first, else go through the store */
-  gtk_tree_model_get (GTK_TREE_MODEL (priv->store), &iter, THUMBNAIL_URL_COLUMN, &path, -1);
-  if (g_ascii_strcasecmp (path, filename))
-  {
-    while (gtk_tree_model_iter_next (GTK_TREE_MODEL (priv->store), &iter))
+    if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->store), &iter))
     {
-      gtk_tree_model_get (GTK_TREE_MODEL (priv->store), &iter, THUMBNAIL_URL_COLUMN, &path, -1);
-      if (!g_ascii_strcasecmp (path, filename))
-      {
-        found = TRUE;
-        break;
-      }
+        gchar *col_filename;
+
+        gtk_tree_model_get (GTK_TREE_MODEL (priv->store), &iter,
+                            THUMBNAIL_URL_COLUMN, &col_filename, -1);
+
+        /* FIXME: col_filename is in GLib filename encoding, not ASCII. */
+        if (g_ascii_strcasecmp (col_filename, filename))
+        {
+            while (gtk_tree_model_iter_next (GTK_TREE_MODEL (priv->store),
+                                             &iter))
+            {
+                g_free (col_filename);
+
+                gtk_tree_model_get (GTK_TREE_MODEL (priv->store), &iter,
+                                    THUMBNAIL_URL_COLUMN, &col_filename, -1);
+
+                /* FIXME: col_filename is in GLib filename encoding, not ASCII. */
+                if (!g_ascii_strcasecmp (col_filename, filename))
+                {
+                    found = TRUE;
+                    break;
+                }
+            }
+        }
+        else
+        {
+            found = TRUE;
+        }
+
+            g_free (col_filename);
+            g_free (filename);
+    }
+    else
+    {
+        /* A single item was in the thumbview but it's been already removed. */
+        g_free (filename);
+        return;
     }
-  }
-  else
-  {
-    found = TRUE;
-  }
-  g_free (path);
-  g_free (filename);
 
   if (!found) return;
 
@@ -406,7 +417,7 @@ cheese_thumb_view_monitor_cb (GFileMonitor     *file_monitor,
       char *photo_name;
       GFile *photo_file;
 
-      priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+      priv = cheese_thumb_view_get_instance_private (thumb_view);
       path_photos = cheese_fileutil_get_photo_path (priv->fileutil);
       filename = g_file_get_path (file);
       if (!strcmp(path_photos, filename))
@@ -424,6 +435,8 @@ cheese_thumb_view_monitor_cb (GFileMonitor     *file_monitor,
         }
         g_dir_close (dir_photos);
       }
+
+      g_free (filename);
       break;
     }
 
@@ -560,7 +573,7 @@ cheese_thumb_view_get_n_selected (CheeseThumbView *thumbview)
 static void
 cheese_thumb_view_fill (CheeseThumbView *thumb_view)
 {
-  CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+  CheeseThumbViewPrivate *priv = cheese_thumb_view_get_instance_private (thumb_view);
 
   GDir       *dir_videos, *dir_photos;
   const char *path_videos, *path_photos;
@@ -627,7 +640,7 @@ static void
 cheese_thumb_view_finalize (GObject *object)
 {
   CheeseThumbView        *thumb_view = CHEESE_THUMB_VIEW (object);
-  CheeseThumbViewPrivate *priv       = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+  CheeseThumbViewPrivate *priv = cheese_thumb_view_get_instance_private (thumb_view);
 
   g_object_unref (priv->store);
   g_object_unref (priv->fileutil);
@@ -646,8 +659,6 @@ cheese_thumb_view_class_init (CheeseThumbViewClass *klass)
 
   object_class->constructed = cheese_thumb_view_constructed;
   object_class->finalize = cheese_thumb_view_finalize;
-
-  g_type_class_add_private (klass, sizeof (CheeseThumbViewPrivate));
 }
 
 static void
@@ -656,7 +667,7 @@ cheese_thumb_view_row_inserted_cb (GtkTreeModel    *tree_model,
                                    GtkTreeIter     *iter,
                                    CheeseThumbView *thumb_view)
 {
-  CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+  CheeseThumbViewPrivate *priv = cheese_thumb_view_get_instance_private (thumb_view);
 
   priv->n_items++;
   if (!priv->vertical)
@@ -670,7 +681,7 @@ cheese_thumb_view_row_deleted_cb (GtkTreeModel    *tree_model,
                                   GtkTreePath     *path,
                                   CheeseThumbView *thumb_view)
 {
-  CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+  CheeseThumbViewPrivate *priv = cheese_thumb_view_get_instance_private (thumb_view);
 
   priv->n_items--;
   if (priv->n_items == 0)
@@ -684,7 +695,7 @@ cheese_thumb_view_row_deleted_cb (GtkTreeModel    *tree_model,
 static void
 cheese_thumb_view_init (CheeseThumbView *thumb_view)
 {
-  CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+  CheeseThumbViewPrivate *priv = cheese_thumb_view_get_instance_private (thumb_view);
 
   priv->video_file_monitor = NULL;
   priv->photo_file_monitor = NULL;
@@ -722,7 +733,7 @@ static void
 cheese_thumb_view_constructed (GObject *object)
 {
   CheeseThumbView *thumb_view = CHEESE_THUMB_VIEW (object);
-  CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+  CheeseThumbViewPrivate *priv = cheese_thumb_view_get_instance_private (thumb_view);
   
   gtk_icon_view_set_model (GTK_ICON_VIEW (thumb_view), GTK_TREE_MODEL (priv->store));
 
@@ -762,7 +773,7 @@ cheese_thumb_view_new ()
 void
 cheese_thumb_view_set_vertical (CheeseThumbView *thumb_view, gboolean vertical)
 {
-  CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+  CheeseThumbViewPrivate *priv = cheese_thumb_view_get_instance_private (thumb_view);
 
   priv->vertical = vertical;
   if (!priv->vertical && priv->n_items)
@@ -774,7 +785,7 @@ cheese_thumb_view_set_vertical (CheeseThumbView *thumb_view, gboolean vertical)
 void
 cheese_thumb_view_start_monitoring_photo_path (CheeseThumbView *thumb_view, const char *path_photos)
 {
-  CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+  CheeseThumbViewPrivate *priv = cheese_thumb_view_get_instance_private (thumb_view);
 
   if (priv->photo_file_monitor != NULL)
     return;
@@ -793,7 +804,7 @@ cheese_thumb_view_start_monitoring_photo_path (CheeseThumbView *thumb_view, cons
 void
 cheese_thumb_view_start_monitoring_video_path (CheeseThumbView *thumb_view, const char *path_videos)
 {
-  CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+  CheeseThumbViewPrivate *priv = cheese_thumb_view_get_instance_private (thumb_view);
 
   if (priv->video_file_monitor != NULL)
     return;
